<?php

// -------------------------------------------------------------
//
// $Id: classes.php,v 1.9 2004/03/29 16:59:27 kuatodotorg Exp $
//
// STARTED   : 02/04/2004
// COPYRIGHT : 2002-2004 kuato.org
// WWW       : http://www.kuato.org/
// LICENSE   : GPL vs2.0 [ see /docs/COPYING ] 
//
// DESCRIPTION  : An RSS module written for Mutant Portal
//
// -------------------------------------------------------------

if( !defined('IN_PORTAL') )
{
    die("Hacking attempt");
}

define('MUT_RSS_SITE_TABLE', $mutcfg->get('mutant_table_prefix').'rss_sites');

//
// This is the base RSS parsing class. 
//
class KO_RssParse
{
	var $rss_version;
	var $content;
	var $curtag;
	var $tag_item;
	var $item_count;
	var $accumulated;
	var $error = "";
	var $parser;
	var $xml = "";

	function KO_RssParse()
	{
		$this->parser = xml_parser_create();

        xml_set_object($this->parser, &$this);
        xml_set_element_handler($this->parser, "_startElement", "_endElement");
        xml_set_character_data_handler($this->parser, "_characterData");

		$this->content = array();
		$this->error = "";
		$this->curtag = "";
		$this->accumulated = "";
		$this->item_count = 0;
		$tihs->tag_item = false;
	}

	// Accessors
	
	function getItemCount()
	{
		return $this->item_count;
	}

	function getContent($tagname)
	{
		return $this->content[$tagname];
	}

	// Mutators 

	function setXml($xml)
	{
		$this->xml = $xml;
	}

	// Manipulators
	function parseXml()
	{
		global $lang;
		// todo: global $user and use $user->lang

		// Create the xml parser and do the work
		if( !xml_parse($this->parser, $this->xml, true) )
		{
			$this->error = $lang['RSS_Error_XML_Parser'];
			$this->error .= ': ' . xml_error_string(xml_get_error_code($this->parser));
			return false;
		}

		return true;
	}

	// Private function
	function _startElement($parser, $name, $attrs) 
	{
		if( $this->tag_item == true )
		{
			$this->curtag = strtolower($name);
		}
		else
		{
			switch( strtolower($name) ) 
			{
				case "item":
				case "article":
					$this->tag_item = true;
					break;
			}
		}
	}

	// Private function
	function _endElement($parser, $name) 
	{
		$name = strtolower($name);

		if( ($name == 'item' || $name == 'article') && ($this->item_count < 5) )
		{
			//print("end of $name <br>");
			$this->tag_item = false;
			$this->curtag = "";
			$this->item_count++;
		}
		else if( $this->tag_item )
		{
			$i 		= $this->item_count;
			$tag 	= $this->curtag;
			//print("tag= $tag  and data= " . $this->accumulated. "<br>");
			$tag = ( $tag == "url") ? "link" : $tag; // for moreover news 
			$this->content['item'][$i][$tag] = $this->accumulated;
			$this->accumulated = "";
		}
	}

	// Private function
	function _characterData($parser, $data) 
	{
		$tag 	= $this->curtag;

		if( ($this->tag_item == true) && ($data = trim($data)) ) 
		{
			//print("tag= $tag  and data= $data<br>");
			$this->accumulated .= $data;
		}
	}
}


//
// Class the provides operations for accessing and setting the data
// in the Rss Sites table
//
class KO_RssSite
{
	// These must match the name and order in the sites database table exactly
	var $fid;
	var $sitename;
	var $siteurl;
	var $numdisplay;
	var $linkorder;
	var $disabled;
	var $cache_interval;
	var $cache_last_update;
	var $xml;
	// Start of other variables that don't need to match order
	var $parser_obj;
	var $initialized = false;
	var $error;

	// Constructor
	function KO_RssSite(&$parser_obj)
	{
		$this->parser_obj =& $parser_obj;
		$initialized = false;
	}

	// Accessors

	function getValue($name)
	{
		return $this->$name;
	}

	// Return a reference to our parser object
	function &getParserObject()
	{
		return $this->parser_obj;
	}

	// Return a copy of our xml data
	function getXml()
	{	
		return $this->xml;
	}

	// Mutators

	function setValue($name, $value)
	{
		$this->$name = $value;
	}

	function assignFid($fid)
	{
		$this->fid = $fid;

		$sql = "SELECT * FROM " . MUT_RSS_SITE_TABLE. "
				 WHERE fid=$this->fid ";

		if( !($result = @$db->sql_query($sql)) )
    	{   
			$rss->error = $lang['RSS_Error_Cache_Table_Read'];
			//$rss->print_err_block();
			return; 
    	}    

		if ($row = $db->sql_fetchrow($result))
		{
			$this->assignRow($row);
		}

		$this->initialized = true;
	}

	function assignRow(&$row)
	{
		if (is_array($row))
		{
			while (list($name,$value) = each($row))
			{
				if (!empty($value) && !is_array($value))
				{
					$this->$name = $value;
				}
			}

			$this->initialized = true;
		}
	}

	//
	// We either insert or update based on if fid was set
	//
	function save()
    {
        global $db, $lang;

		// always update cache_last_update
		$this->cache_last_update = time();

		// only add slashes if runtime magic quotes is off
		if( get_magic_quotes_runtime() == 0 )
		{
			$this->xml = addslashes($this->xml);
		}

		if (empty($this->fid))
		{
			$sql = "INSERT INTO " . MUT_RSS_SITE_TABLE. "
					 VALUES (NULL, '$this->sitename', $this->siteurl,  '$this->numdisplay', '$this->linkorder', '$this->disabled', '$this->cache_interval', '$this->cache_last_update', '$this->xml')";
			if( !($result = $db->sql_query($sql)) )
			{
				$this->error = $lang['RSS_Error_Site_Table_Write'];
				return false;
			}
		}
		else
		{
			if ($this->initialized == true)
			{
				$sql = "UPDATE " . MUT_RSS_SITE_TABLE. "
						SET sitename='$this->sitename', siteurl='$this->siteurl', numdisplay='$this->numdisplay', linkorder='$this->linkorder', disabled='$this->disabled', cache_interval='$this->cache_interval', cache_last_update='$this->cache_last_update', xml='$this->xml' WHERE fid='$this->fid' ";

				if( !($result = $db->sql_query($sql)) )
				{
					$this->error = $lang['RSS_Error_Site_Table_Write'];
					return false;
				}
			}
			else 
			{
				return false;
			}
		}

		// only strip slashes if runtime magic quotes is off
        if( get_magic_quotes_runtime() == 0 )
        {
            $this->xml = stripslashes($this->xml);
        }

        return true;
    }

	// Output errors to the block template
	function print_err_block()
	{
		global $template;

		if( !empty($this->error) )
		{
			$this->error = '<font size="2" color="red">' . 'Error: ' . $this->error . '</font>';
		}

		// Fill in error for this row
		$template->assign_block_vars("mut_rssrow", array(
			'MUT_RSS_URL'      => $this->error
		));
	}

	//
	// Get the XML newsfeed data stream from the server
	// and save
	//
	// returns: false if update interval hasn't been reached
	//		    true if time interval has been reached or exceeded whether
	//			  or not there was an error in connecting to server
	//          We always return even if there's an error in connecting 
	//
	function updateCache()
	{
		global $module_root_path, $portal_config, $lang;

		$interval = $this->cache_interval * 60;
		$last_update = $this->cache_last_update;

		if( (time() - $last_update) < $interval)
		{
			return false;
		}

		$rss = parse_url($this->siteurl);
		if( !isset($rss['port']) ) 
		{
			$rss['port'] = 80;
		}
		if( !isset($rss['query']) )
		{
			$rss['query'] = ''; 
		}

		$fpread = @fsockopen($rss['host'], $rss['port'], &$errno, &$this->error, 30);
		if( $fpread )
		{
			//
			// Request the XML file from the server
			//
			$query = (isset($rss['query'])) ? $rss['query'] : '';

			$request = 'GET ' .  $rss['path'] . '?' . $query . " HTTP/1.0\r\n";
			$request .= 'HOST: '.  $rss['host']. "\r\n";
			$request .= 'Referer: ' .$portal_config[portal_url]. "\r\n";
			$request .= 'User-Agent: ' . 'mutant' . "\r\n\r\n";
			//socket_set_timeout($fpread, 10);
			fputs($fpread, $request);

			$start_time = time();

			//
			// Get the XML data and strip out the header info from 
			// the data stream
			//
			while ( (!feof($fpread)) ) 
			{
				$tmpline = @fgets($fpread, 4096);
				$line .= $tmpline;

				if( (time() - $start_time) >= 5) 
				{
					$this->error = $lang['RSS_Error_Server_Timed_Out'];
					return true;
				}
			}

			if( preg_match ("/<rss version=/i", $line) )
			{
				$this->xml = stristr($line, "<rss");
				$this->rss_version = "new";
			}
			else if( preg_match ("/<rdf:RDF/i", $line) )
			{
				$this->xml = stristr($line, "<rdf:");
				$this->rss_version = "old";
			}
			else if( preg_match ("/<moreovernews/i", $line) )
			{
				$this->xml = stristr($line, "<moreovernews>");
				$this->rss_version = "moreover";
			}

			// Close file
			fputs($fpread, "Connection: close\r\n\r\n");
			fclose($fpread);
		}
		else
		{
			$this->error = $lang['RSS_Error_Server_Open'];
			return true;
		}

		return true;
	}
}

?>
