<?php

//-------------------------------------------------------------
//
// $Id: rss.php,v 1.6 2004/04/04 09:33:27 kuatodotorg Exp $
//
// STARTED   : 02/04/2004
// COPYRIGHT : 2002-2004 kuato.org
// WWW       : http://www.kuato.org/
// LICENSE   : GPL vs2.0 [ see /docs/COPYING ] 
//
// FUNCTION	 : Create an RSS 2.0 newsfeed from our news or article database table 
//		       depending on the id passed in. 
// TODO 	 : Ignore clients that don't behave 
//			   Verify id passed in belongs to the news or articles tree
//
//-------------------------------------------------------------

// Variable defaults
$phpbb_root_path = '../../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);

// Defines
define('IN_PHPBB', true);

// Includes
include($phpbb_root_path . 'common.'.$phpEx);
include($phpbb_root_path . '/mutant_includes/mutant_functions.'.$phpEx);
include($phpbb_root_path . '/mutant_includes/class.config.'.$phpEx);
include($phpbb_root_path . '/mutant_includes/class.mutant_forum_tree.'.$phpEx);

// Get config singleton
$mutcfg =& MutantConfig::getInstance();

// Will need guest user created to for later to verify branches of a news
// or articles tree are viewable.
$user->start();
$auth->acl($user->data);
$user->setup();

// Parse uri
$forum_id	= isset($_REQUEST['id']) ? (int)$_REQUEST['id'] : 0;

// Channel link
$link = (preg_match("/^http/", $config['server_name'])) ? "" : 'http://';
$link .= $config['server_name'] . $config['script_path'] . '/';

// Start the xml header
$xml .= "<?xml version=\"1.0 encoding=\"utf-8\" ?>\r\n";
$xml .= "<rss version=\"2.0\">\r\n";
$xml .= "<channel>\r\n";
$xml .= "	<title>" . $config['sitename'] . "</title>\r\n";
$xml .= "	<description>" . $config['site_desc'] . "</description>\r\n";
$xml .= "	<link>" . $link . "</link>\r\n"; 

// Safety checks
if( $forum_id == 0 )
{
	die('invalid id');
}

// TODO: Verify that forum_id is valid news/articles root_id or at least one of
// its children.
// The temporay solution is to only allow from news_root_id and articles_root_id
if( ($forum_id != $mutcfg->get('news_root_id')) && ($forum_id != $mutcfg->get('articles_root_id')) )
{
	die('id is not a valid news or article id');
}

// Grab the forum tree 
$forum_tree =& new MutantForumTree($db, $forum_id);

$forum_list =& $forum_tree->createForumList($forum_id, $forum_id);
if( !($forum_list) )
{
    return;
}

$nbr_display = 10;

$sql = 'SELECT 
			f.forum_id, f.forum_name,
			t.topic_id, t.topic_replies, t.topic_views, t.topic_title, 
				t.topic_last_post_id, t.forum_id,
			p.post_time, p.post_text, p.bbcode_uid, p.bbcode_bitfield,
			u.username, u.user_id
		FROM 
			'.FORUMS_TABLE.' f  
		INNER JOIN
			'.TOPICS_TABLE.' t 
			ON f.forum_id = t.forum_id
		INNER JOIN
			'.POSTS_TABLE.' p 
			ON t.topic_first_post_id = p.post_id
		INNER JOIN
			'.USERS_TABLE.' u 
			ON p.poster_id = u.user_id
		WHERE 	p.topic_id = t.topic_id 
			AND t.topic_type in (' .POST_NORMAL. ' ) 
			AND t.topic_moved_id = 0
			AND t.topic_approved = 1
			AND t.forum_id IN (' . implode(', ', $forum_list) . ")
		ORDER BY p.post_time DESC";
$result = $db->sql_query_limit($sql, $nbr_display, 0);

// Process each row into an RSS 2.0 format of title, link, description, pubDate
while( $row = $db->sql_fetchrow($result) )
{
	$title      =& $row['topic_title'];
	$message    =& $row['post_text'];
	$forum_id	=& $row['forum_id'];
	$topic_id	=& $row['topic_id'];
	$bbcode_uid 		= $row['bbcode_uid'];
    $bbcode_bitfield    = $row['bbcode_bitfield'];

	//$post_date  = $user->format_date($row['post_time'], 'D M d, Y');
	// TODO: adjust for config['board_timezone']
	$post_date  = gmdate('D M d, Y', $row['post_time']);
	$message = mutant_decode_message($message, $bbcode_uid, $bbcode_bitfield);

	// We need to make sure to always chop the message so we don't return any
	// bbcodes by mistake.
	$tease_len = 500;
	$message = mutant_chop_message($message, $tease_len);
	$message = htmlspecialchars($message);

	// Now add the item, title, link, description, pubDate for this row to
	// the xml feed output.
	$xml .= "	<item>\r\n";
    $xml .= "		<title>" . $title. "</title>\r\n";
	// TODO: Need to add appropiate page name to link based on whether this forum
	// id is news or articles
    $xml .= "		<link>" . $link. "?f=$forum_id&amp;view=story&amp;t=$topic_id" . "</link>\r\n";
    $xml .= "		<description>" . $message. "</description>\r\n";
    $xml .= "		<pubDate>" . $post_date . "</pubDate>\r\n";
    // TODO $xml .= "		<guid>" . $basehref . "/?page=comments&amp;action=read&amp;article=" . $row->id . "</guid>\r\n";
    $xml .= "	</item>\r\n\r\n"; 
}

$xml .= "</channel>\r\n";
$xml .= "</rss>\r\n"; 

echo $xml;

?>
